/**
 * Set headers for a request.
 */
function setHeaders(aRequest) {
  aRequest.setRequestHeader("User-Agent", navigator.userAgent + " delicious.mozdev.org");
  return aRequest;
}

/**
 * Retrieve an URL synchronously. Note that this is synchronous, that is, it
 * will block (stall the browser!) until it's done downloading the entire
 * document.
 */
function getURL(url, username, password, asXML)
{
    var request = new XMLHttpRequest();
    
    // Retrieve page
    request.open("GET", url, false, username, password);
    setHeaders(request);

    // Basic authentication should be used
    if(username.length > 0)
    {
        var text = username + ":" + password;
        text = encodeBase64(text);
        request.setRequestHeader("Authorization", "Basic " + text);
    }
    request.send(null);

    if (request.readyState == 4 && request.status == 200)
    {
    	if(asXML) return request.responseXML;    	
    	else return request.responseText;
    }
    else throw("Unable to get URL '" + url + "' status code " + request.status);
}

/**
 * Retrieve an URL asynchronously.
 * The asyncGetURLRequester given as a parameter will be called when the URL has been
 * loaded successfully or when an error has happened.
 */
function asyncGetURL(url, username, password, asXML, asyncGetURLRequester)
{
    var request = new XMLHttpRequest();

    // Open an asynchronous request
    request.open("GET", url, true, username, password);
    request.overrideMimeType('text/xml');
    setHeaders(request);

    // Basic authentication should be used
    if(username.length > 0)
    {
        var text = username + ":" + password;
        text = encodeBase64(text);
        request.setRequestHeader("Authorization", "Basic " + text);
    }
    request.onreadystatechange = function() {
        // 4 is the "completed" state
        if(request.readyState==4) {
            switch(request.status) {
                case 401:
                    alert("Del.icio.us couldn't authorize this operation. Please check the used account's username and password and retry.");
                    if (asyncGetURLRequester) {
                        asyncGetURLRequester.asyncGetURLError(401);
                    }
                    break;
                case 200:
                    // HTTP "OK". Successfully posted.
                    if(asyncGetURLRequester) {
                        if(asXML) {
                            asyncGetURLRequester.asyncGetURLResponse(request.responseXML);
                        } else {
                            asyncGetURLRequester.asyncGetURLResponse(request.responseText);
                        }
                    } 
                    break;
                default:
                    if (asyncGetURLRequester) {
                        asyncGetURLRequester.asyncGetURLError(null);
                    }
                    break;
            }
        } 
    };
    request.send(null);
}

/**
 * Post an URL asynchronously.
 * The asyncGetURLRequester given as a parameter will be called when the URL has been
 * loaded successfully or when an error has happened.
 */
function asyncPostURL(url, username, password, asXML, asyncGetURLRequester, data)
{
    var request = new XMLHttpRequest();

    // Open an asynchronous request
    request.open("POST", url, true);
    setHeaders(request);

    // Basic authentication should be used
    if(username.length > 0)
    {
        var text = username + ":" + password;
        text = encodeBase64(text);
        request.setRequestHeader("Authorization", "Basic " + text);
    }
    request.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
    request.onreadystatechange = function() {
        // 4 is the "completed" state
        if(request.readyState==4) {
            switch(request.status) {
                case 401:
                    alert("Del.icio.us couldn't authorize this operation. Please check the used account's username and password and retry.");
                    if (asyncGetURLRequester) {
                        asyncGetURLRequester.asyncGetURLError(401);
                    }
                    break;
                case 200:
                    // HTTP "OK". Successfully posted.
                    if(asyncGetURLRequester) {
                        if(asXML) {
                            asyncGetURLRequester.asyncGetURLResponse(request.responseXML);
                        } else {
                            asyncGetURLRequester.asyncGetURLResponse(request.responseText);
                        }
                    }
                    break;
                default:
                    if (asyncGetURLRequester) {
                        asyncGetURLRequester.asyncGetURLError(null);
                    }
                    break;
            }
        }
    };
    request.send(data);
}
